'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _debug = require('./debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (Model, options) {
  options = (0, _assign2.default)({ _id: '_id', _from: '_from', _to: '_to' }, options);
  (0, _debug2.default)('Edge mixin for Model %s with options %O', Model.modelName, options);

  Model.defineProperty(options._id, { type: String, _id: true });
  Model.defineProperty(options._from, {
    type: String,
    _from: true,
    required: true
  });
  Model.defineProperty(options._to, { type: String, _to: true, required: true });

  /**
   * Build data from database to relative model
   * @param connector The connector
   * @param edges Array<model> The edges data
   * @returns {Array<Model>}
   */
  function buildModelFromDatabase(connector, edges) {
    var modelName = Model.modelName;
    var idName = connector.idName(modelName);
    var fullIdName = connector._fullIdName(modelName);
    var fromName = connector._fromName(modelName);
    var toName = connector._toName(modelName);
    edges = edges.map(function (edge) {
      delete edge._rev;

      connector._setFieldValue(edge, idName, edge._key);
      if (idName !== '_key') delete edge._key;

      connector._setFieldValue(edge, fullIdName, edge._id);
      if (fullIdName !== '_id') delete edge._id;

      connector._setFieldValue(edge, fromName, edge._from);
      if (fromName !== '_from') delete edge._from;

      connector._setFieldValue(edge, toName, edge._to);
      if (toName !== '_to') delete edge._to;

      edge = connector.fromDatabase(modelName, edge);

      return new Model(edge);
    });
    return edges;
  }

  /**
   * Retrieves a list of all edges of the document with the given nodeId.
   * @param nodeId {String} The _id property of arangoDB document
   * @param options {Object} The options object
   * @returns {Promise.<Model|Error>}
   */
  Model.edges = function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(nodeId, options) {
      var modelName, connector, collection, edges;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              modelName = Model.modelName;
              connector = this.getConnector();
              collection = connector.getCollection(modelName);
              _context.next = 5;
              return collection.edges(nodeId);

            case 5:
              edges = _context.sent;
              return _context.abrupt('return', buildModelFromDatabase(connector, edges));

            case 7:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    return function (_x, _x2) {
      return _ref.apply(this, arguments);
    };
  }();

  /**
   * Retrieves a list of all incoming edges of the document with the given nodeId.
   * @param nodeId {String} The _id property of arangoDB document
   * @param options {Object}
   * @returns {Promise.<Model|Error>}
   */
  Model.inEdges = function () {
    var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(nodeId, options) {
      var modelName, connector, collection, edges;
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              modelName = Model.modelName;
              connector = this.getConnector();
              collection = connector.getCollection(modelName);
              _context2.next = 5;
              return collection.inEdges(nodeId);

            case 5:
              edges = _context2.sent;
              return _context2.abrupt('return', buildModelFromDatabase(connector, edges));

            case 7:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    return function (_x3, _x4) {
      return _ref2.apply(this, arguments);
    };
  }();

  /**
   * Retrieves a list of all outgoing edges of the document with the given nodeId.
   * @param nodeId {String} The _id property of arangoDB document
   * @param options {Object} The options object
   * @returns {Promise.<Model|Error>}
   */
  Model.outEdges = function () {
    var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(nodeId, options) {
      var modelName, connector, collection, edges;
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              modelName = Model.modelName;
              connector = this.getConnector();
              collection = connector.getCollection(modelName);
              _context3.next = 5;
              return collection.outEdges(nodeId);

            case 5:
              edges = _context3.sent;
              return _context3.abrupt('return', buildModelFromDatabase(connector, edges));

            case 7:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, this);
    }));

    return function (_x5, _x6) {
      return _ref3.apply(this, arguments);
    };
  }();

  /**
   * Performs a traversal starting from the given startVertex and following
   * edges contained in this edge collection.
   *
   * @param graphOptions {Object} The graph options
   * @property {String} startVertex The start vertex
   * @param options {Object} The options object
   * @returns {Promise.<Model|Error>}
   */
  Model.traversal = function () {
    var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(graphOptions, options) {
      var err, startVertex, modelName, connector, collection;
      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              (0, _assert2.default)(typeof graphOptions === 'object', 'graphOptions must be an object');

              if (!(typeof graphOptions.startVertex !== 'string')) {
                _context4.next = 8;
                break;
              }

              err = new Error('startVertex is required');

              err.code = 'START_VERTEX_IS_REQUIRED';
              err.statusCode = 400;
              return _context4.abrupt('return', _bluebird2.default.reject(err));

            case 8:
              startVertex = graphOptions.startVertex;
              modelName = Model.modelName;
              connector = this.getConnector();
              collection = connector.getCollection(modelName);
              _context4.next = 14;
              return collection.traversal(startVertex, graphOptions);

            case 14:
              return _context4.abrupt('return', _context4.sent);

            case 15:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, this);
    }));

    return function (_x7, _x8) {
      return _ref4.apply(this, arguments);
    };
  }();

  if (typeof Model.remoteMethod === 'function') {
    Model.remoteMethod('edges', {
      description: `Retrieves a list of all ${Model.modelName} edges of the 
        document with the given documentHandle.`,
      accepts: [{
        arg: 'nodeId',
        type: 'string',
        description: 'The _id property of arangoDB document.',
        http: { source: 'query' },
        required: true
      }, { arg: 'options', type: 'object', http: 'optionsFromRequest' }],
      http: { path: '/edges', verb: 'get' },
      returns: { arg: 'data', root: true, type: [Model.modelName] }
    });

    Model.remoteMethod('inEdges', {
      description: `Retrieves a list of all incoming ${Model.modelName} edges 
        of the document with the given documentHandle.`,
      accepts: [{
        arg: 'nodeId',
        type: 'string',
        description: 'The _id property of arangoDB document.',
        http: { source: 'query' },
        required: true
      }, { arg: 'options', type: 'object', http: 'optionsFromRequest' }],
      http: { path: '/inEdges', verb: 'get' },
      returns: { arg: 'data', root: true, type: [Model.modelName] }
    });

    Model.remoteMethod('outEdges', {
      description: `Retrieves a list of all outgoing ${Model.modelName} edges
        of the document with the given documentHandle.`,
      accepts: [{
        arg: 'nodeId',
        type: 'string',
        description: 'The _id property of arangoDB document.',
        http: { source: 'query' },
        required: true
      }, { arg: 'options', type: 'object', http: 'optionsFromRequest' }],
      http: { path: '/outEdges', verb: 'get' },
      returns: { arg: 'data', root: true, type: [Model.modelName] }
    });

    Model.remoteMethod('traversal', {
      description: `Performs a traversal starting from the given startVertex 
        and following edges contained in ${Model.modelName} edge collection.`,
      accepts: [{
        arg: 'graphOptions',
        type: 'object',
        http: { source: 'query' },
        required: true
      }, { arg: 'options', type: 'object', http: 'optionsFromRequest' }],
      http: { path: '/traversal', verb: 'get' },
      returns: { arg: 'result', root: true, type: 'object' }
    });
  }
};